@@ -1,5 +1,5 @@
 """Top-level package for Bayesian estimation of proportions."""
 
 __author__ = """Oliver Chipperfield"""
-__email__ = 'omc0dev@googlemail.com'
-__version__ = '0.1.7'
+__email__ = "omc0dev@googlemail.com"
+__version__ = "0.9.1"
